@@ -1,16 +1,12 @@
 /*
- * $Header: /home/cvs/jakarta-commons/validator/src/share/org/apache/commons/validator/ValidatorAction.java,v 1.20.2.1 2004/04/13 05:49:22 rleland Exp $
- * $Revision: 1.20.2.1 $
- * $Date: 2004/04/13 05:49:22 $
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
- * ====================================================================
- * Copyright 2001-2004 The Apache Software Foundation
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -18,7 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.commons.validator;
 
 import java.io.BufferedReader;
@@ -30,7 +25,6 @@ import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.util.ArrayList;
-import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
@@ -46,13 +40,15 @@ import org.apache.commons.validator.util.ValidatorUtils;
  * be defined in an xml file with the &lt;validator&gt; element.
  *
  * <strong>Note</strong>: The validation method is assumed to be thread safe.
+ *
+ * @version $Revision: 478334 $ $Date: 2006-11-22 21:31:54 +0000 (Wed, 22 Nov 2006) $
  */
 public class ValidatorAction implements Serializable {
     
     /**
      * Logger.
      */
-    private static final Log log = LogFactory.getLog(ValidatorAction.class);
+    private transient Log log = LogFactory.getLog(ValidatorAction.class);
 
     /**
      * The name of the validation.
@@ -161,6 +157,7 @@ public class ValidatorAction implements Serializable {
 
     /**
      * Gets the name of the validator action.
+     * @return Validator Action name.
      */
     public String getName() {
         return name;
@@ -168,6 +165,7 @@ public class ValidatorAction implements Serializable {
 
     /**
      * Sets the name of the validator action.
+     * @param name Validator Action name.
      */
     public void setName(String name) {
         this.name = name;
@@ -175,6 +173,7 @@ public class ValidatorAction implements Serializable {
 
     /**
      * Gets the class of the validator action.
+     * @return Class name of the validator Action.
      */
     public String getClassname() {
         return classname;
@@ -182,6 +181,7 @@ public class ValidatorAction implements Serializable {
 
     /**
      * Sets the class of the validator action.
+     * @param classname Class name of the validator Action.
      */
     public void setClassname(String classname) {
         this.classname = classname;
@@ -189,6 +189,7 @@ public class ValidatorAction implements Serializable {
 
     /**
      * Gets the name of method being called for the validator action.
+     * @return The method name.
      */
     public String getMethod() {
         return method;
@@ -196,6 +197,7 @@ public class ValidatorAction implements Serializable {
 
     /**
      * Sets the name of method being called for the validator action.
+     * @param method The method name.
      */
     public void setMethod(String method) {
         this.method = method;
@@ -203,6 +205,7 @@ public class ValidatorAction implements Serializable {
 
     /**
      * Gets the method parameters for the method.
+     * @return Method's parameters.
      */
     public String getMethodParams() {
         return methodParams;
@@ -227,17 +230,10 @@ public class ValidatorAction implements Serializable {
         }
     }
 
-    /**
-     * Gets the method parameters for the method as an unmodifiable List.
-     * @deprecated This will be removed after Validator 1.1.2
-     */
-    public List getMethodParamsList() {
-        return Collections.unmodifiableList(this.methodParameterList);
-    }
-
     /**
      * Gets the dependencies of the validator action as a comma separated list 
      * of validator names.
+     * @return The validator action's dependencies.
      */
     public String getDepends() {
         return this.depends;
@@ -264,6 +260,7 @@ public class ValidatorAction implements Serializable {
 
     /**
      * Gets the message associated with the validator action.
+     * @return The message for the validator action.
      */
     public String getMsg() {
         return msg;
@@ -271,6 +268,7 @@ public class ValidatorAction implements Serializable {
 
     /**
      * Sets the message associated with the validator action.
+     * @param msg The message for the validator action.
      */
     public void setMsg(String msg) {
         this.msg = msg;
@@ -280,6 +278,7 @@ public class ValidatorAction implements Serializable {
      * Gets the Javascript function name.  This is optional and can
      * be used instead of validator action name for the name of the
      * Javascript function/object.
+     * @return The Javascript function name.
      */
     public String getJsFunctionName() {
         return jsFunctionName;
@@ -289,6 +288,7 @@ public class ValidatorAction implements Serializable {
      * Sets the Javascript function name.  This is optional and can
      * be used instead of validator action name for the name of the
      * Javascript function/object.
+     * @param jsFunctionName The Javascript function name.
      */
     public void setJsFunctionName(String jsFunctionName) {
         this.jsFunctionName = jsFunctionName;
@@ -318,6 +318,7 @@ public class ValidatorAction implements Serializable {
      *         org.apache.commons.validator.javascript.validateTire.js
      *      which is the default javascript definition.
      * </pre>
+     * @param jsFunction The Javascript function's fully qualified class path.
      */
     public void setJsFunction(String jsFunction) {
         if (javascript != null) {
@@ -330,6 +331,7 @@ public class ValidatorAction implements Serializable {
     /**
      * Gets the Javascript equivalent of the java class and method
      * associated with this action.
+     * @return The Javascript validation.
      */
     public String getJavascript() {
         return javascript;
@@ -338,6 +340,7 @@ public class ValidatorAction implements Serializable {
     /**
      * Sets the Javascript equivalent of the java class and method
      * associated with this action.
+     * @param javascript The Javascript validation.
      */
     public void setJavascript(String javascript) {
         if (jsFunction != null) {
@@ -347,22 +350,6 @@ public class ValidatorAction implements Serializable {
         this.javascript = javascript;
     }
 
-    /**
-     * Gets an instance based on the validator action's classname.
-     * @deprecated This will be removed after Validator 1.1.2
-     */
-    public Object getClassnameInstance() {
-        return instance;
-    }
-
-    /**
-     * Sets an instance based on the validator action's classname.
-     * @deprecated This will be removed after Validator 1.1.2
-     */
-    public void setClassnameInstance(Object instance) {
-        this.instance = instance;
-    }
-
     /**
      * Initialize based on set.
      */
@@ -387,8 +374,8 @@ public class ValidatorAction implements Serializable {
             return;
         }
 
-        if (log.isTraceEnabled()) {
-            log.trace("  Loading function begun");
+        if (getLog().isTraceEnabled()) {
+            getLog().trace("  Loading function begun");
         }
 
         if (this.jsFunction == null) {
@@ -397,14 +384,14 @@ public class ValidatorAction implements Serializable {
 
         String javascriptFileName = this.formatJavascriptFileName();
 
-        if (log.isTraceEnabled()) {
-            log.trace("  Loading js function '" + javascriptFileName + "'");
+        if (getLog().isTraceEnabled()) {
+            getLog().trace("  Loading js function '" + javascriptFileName + "'");
         }
 
         this.javascript = this.readJavascriptFile(javascriptFileName);
 
-        if (log.isTraceEnabled()) {
-            log.trace("  Loading javascript function completed");
+        if (getLog().isTraceEnabled()) {
+            getLog().trace("  Loading javascript function completed");
         }
 
     }
@@ -426,7 +413,7 @@ public class ValidatorAction implements Serializable {
         }
 
         if (is == null) {
-            log.debug("  Unable to read javascript name "+javascriptFileName);
+            getLog().debug("  Unable to read javascript name "+javascriptFileName);
             return null;
         }
 
@@ -439,13 +426,13 @@ public class ValidatorAction implements Serializable {
             }
 
         } catch(IOException e) {
-            log.error("Error reading javascript file.", e);
+            getLog().error("Error reading javascript file.", e);
 
         } finally {
             try {
                 reader.close();
             } catch(IOException e) {
-                log.error("Error closing stream to javascript file.", e);
+                getLog().error("Error closing stream to javascript file.", e);
             }
         }
         
@@ -488,34 +475,19 @@ public class ValidatorAction implements Serializable {
         return jsName.toString();
     }
 
-    /**
-     * Creates a <code>FastHashMap</code> for the isDependency method
-     * based on depends.
-     * @deprecated This functionality has been moved to other methods.  It's no 
-     * longer required to call this method to initialize this object.
-     */
-    public synchronized void process(Map globalConstants) {
-        // do nothing
-    }
-
     /**
      * Checks whether or not the value passed in is in the depends field.
+     * @param validatorName Name of the dependency to check.
+     * @return Whether the named validator is a dependant.
      */
     public boolean isDependency(String validatorName) {
         return this.dependencyList.contains(validatorName);
     }
 
-    /**
-     * Gets the dependencies as a <code>Collection</code>.
-     * @deprecated Use getDependencyList() instead.
-     */
-    public Collection getDependencies() {
-        return this.getDependencyList();
-    }
-
     /**
      * Returns the dependent validator names as an unmodifiable
      * <code>List</code>.
+     * @return List of the validator action's depedents.
      */
     public List getDependencyList() {
         return Collections.unmodifiableList(this.dependencyList);
@@ -523,6 +495,7 @@ public class ValidatorAction implements Serializable {
 
     /**
      * Returns a string representation of the object.
+     * @return a string representation.
      */
     public String toString() {
         StringBuffer results = new StringBuffer("ValidatorAction: ");
@@ -551,10 +524,14 @@ public class ValidatorAction implements Serializable {
         params.put(Validator.VALIDATOR_ACTION_PARAM, this);
 
         try {
-            ClassLoader loader = this.getClassLoader(params);
-            this.loadValidationClass(loader);
-            this.loadParameterClasses(loader);
-            this.loadValidationMethod();
+            if (this.validationMethod == null) {
+                synchronized(this) {
+                    ClassLoader loader = this.getClassLoader(params);
+                    this.loadValidationClass(loader);
+                    this.loadParameterClasses(loader);
+                    this.loadValidationMethod();
+                }
+            }
 
             Object[] paramValues = this.getParameterValues(params);
             
@@ -599,7 +576,7 @@ public class ValidatorAction implements Serializable {
                 throw (ValidatorException) e;
             }
 
-            log.error(
+            getLog().error(
                 "Unhandled exception thrown during validation: " + e.getMessage(),
                 e);
 
@@ -624,7 +601,8 @@ public class ValidatorAction implements Serializable {
                 this.validationClass.getMethod(this.method, this.parameterClasses);
      
         } catch (NoSuchMethodException e) {
-            throw new ValidatorException(e.getMessage());
+            throw new ValidatorException("No such validation method: " + 
+                e.getMessage());
         }
     }
     
@@ -643,7 +621,7 @@ public class ValidatorAction implements Serializable {
         try {
             this.validationClass = loader.loadClass(this.classname);
         } catch (ClassNotFoundException e) {
-            throw new ValidatorException(e.getMessage());
+            throw new ValidatorException(e.toString());
         }
     }
     
@@ -661,18 +639,20 @@ public class ValidatorAction implements Serializable {
             return;
         }
         
-        this.parameterClasses = new Class[this.methodParameterList.size()];
+        Class[] parameterClasses = new Class[this.methodParameterList.size()];
 
         for (int i = 0; i < this.methodParameterList.size(); i++) {
             String paramClassName = (String) this.methodParameterList.get(i);
 
             try {
-                this.parameterClasses[i] = loader.loadClass(paramClassName);
+                parameterClasses[i] = loader.loadClass(paramClassName);
                     
             } catch (ClassNotFoundException e) {
                 throw new ValidatorException(e.getMessage());
             }
         }
+
+        this.parameterClasses = parameterClasses;
     }
     
     /**
@@ -793,4 +773,20 @@ public class ValidatorAction implements Serializable {
         return v.getOnlyReturnErrors();
     }
 
-}
\ No newline at end of file
+    /**
+     * Accessor method for Log instance.
+     *
+     * The Log instance variable is transient and
+     * accessing it through this method ensures it
+     * is re-initialized when this instance is
+     * de-serialized.
+     *
+     * @return The Log instance.
+     */
+    private Log getLog() {
+        if (log == null) {
+            log =  LogFactory.getLog(ValidatorAction.class);
+        }
+        return log;
+    }
+}
